package com.tzld.piaoquan.api.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.growth.common.common.enums.GhTypeEnum;
import com.tzld.piaoquan.growth.common.common.enums.StrategyStatusEnum;
import com.tzld.piaoquan.growth.common.dao.mapper.GhDetailMapper;
import com.tzld.piaoquan.growth.common.model.po.GhDetail;
import com.tzld.piaoquan.growth.common.model.po.GhDetailExample;
import com.tzld.piaoquan.api.model.vo.GhDetailVo;
import com.tzld.piaoquan.api.service.GhDetailService;
import com.tzld.piaoquan.growth.common.common.base.CommonResponse;
import com.tzld.piaoquan.growth.common.utils.DateUtil;
import com.tzld.piaoquan.growth.common.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class GhDetailServiceImpl implements GhDetailService {

    @Autowired
    private GhDetailMapper ghDetailMapper;


    @Override
    public CommonResponse<Page<GhDetailVo>> getGhDetailList(Integer pageNum, Integer pageSize, String accountId) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        if (pageSize > 100) {
            pageSize = 100;
        }
        Page<GhDetailVo> page = new Page<>();
        page.setCurrentPage(pageNum);
        page.setPageSize(pageSize);
        GhDetailExample example = new GhDetailExample();
        example.setOrderByClause("create_time desc");
        GhDetailExample.Criteria criteria = example.createCriteria().andIsDeleteEqualTo(0);
        if (StringUtils.isNotEmpty(accountId)) {
            criteria.andGhIdEqualTo(accountId);
        }
        example.setPage(page);
        long total = ghDetailMapper.countByExample(example);
        page.setTotalSize((int) total);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        List<GhDetailVo> ghDetailVos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(ghDetails)) {
            for (GhDetail ghDetail : ghDetails) {
                GhDetailVo ghDetailVo = new GhDetailVo();
                BeanUtils.copyProperties(ghDetail, ghDetailVo);
                ghDetailVo.setAccountId(ghDetail.getGhId());
                ghDetailVo.setAccountName(ghDetail.getGhName());
                ghDetailVo.setTypeName(GhTypeEnum.getTypeName(ghDetailVo.getType()));
                ghDetailVo.setCreateTime(DateUtil.getDateString(ghDetail.getCreateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
                ghDetailVo.setUpdateTime(DateUtil.getDateString(ghDetail.getUpdateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
                ghDetailVo.setChannel(ghDetail.getChannel());
                if (StringUtils.isNotEmpty(ghDetail.getVideoIds())) {
                    List<Long> videoIds = JSONObject.parseArray(ghDetail.getVideoIds(), Long.class);
                    ghDetailVo.setVideoIds(videoIds);
                }
                ghDetailVo.setStrategyStatusName(StrategyStatusEnum.getStrategyStatusName(ghDetail.getStrategyStatus()));
                ghDetailVos.add(ghDetailVo);
            }
        }
        page.setObjs(ghDetailVos);
        return CommonResponse.success(page);
    }


    @Override
    public CommonResponse<Void> addGhDetail(GhDetailVo ghDetailVo) {
        if (ghDetailVo == null
                || StringUtils.isEmpty(ghDetailVo.getAccountId())
                || StringUtils.isEmpty(ghDetailVo.getAccountName())
                || ghDetailVo.getType() == null
                || StringUtils.isEmpty(ghDetailVo.getCategory1())) {
            return CommonResponse.create(500, "参数错误");
        }

        try {
            GhDetail ghDetail = new GhDetail();
            BeanUtils.copyProperties(ghDetailVo, ghDetail);
            if (StringUtils.isNotEmpty(ghDetailVo.getAccountId())) {
                ghDetail.setGhId(ghDetailVo.getAccountId().trim());
            }
            if (StringUtils.isNotEmpty(ghDetailVo.getAccountName())) {
                ghDetail.setGhName(ghDetailVo.getAccountName());
            }
            ghDetail.setGhId(ghDetailVo.getAccountId());
            ghDetail.setGhName(ghDetailVo.getAccountName());
            if (!CollectionUtils.isEmpty(ghDetailVo.getVideoIds())) {
                ghDetail.setVideoIds(JSONObject.toJSONString(ghDetailVo.getVideoIds()));
            }
            ghDetailMapper.insertSelective(ghDetail);
            return CommonResponse.success();
        } catch (Exception e) {
            log.error("addGhDetail error", e);
        }
        return CommonResponse.create(500, "插入失败，可能已经存在该记录");
    }

    @Override
    public CommonResponse<Void> updateDetail(GhDetailVo ghDetailVo) {
        try {
            GhDetail ghDetail = new GhDetail();
            BeanUtils.copyProperties(ghDetailVo, ghDetail);
            if (StringUtils.isNotEmpty(ghDetailVo.getAccountId())) {
                ghDetail.setGhId(ghDetailVo.getAccountId().trim());
            }
            if (StringUtils.isNotEmpty(ghDetailVo.getAccountName())) {
                ghDetail.setGhName(ghDetailVo.getAccountName());
            }
            ghDetail.setGhId(ghDetailVo.getAccountId());
            ghDetail.setGhName(ghDetailVo.getAccountName());
            ghDetail.setVideoIds(JSONObject.toJSONString(ghDetailVo.getVideoIds()));
            ghDetailMapper.updateByPrimaryKeySelective(ghDetail);
            return CommonResponse.success();
        } catch (Exception e) {
            log.error("updateDetail error", e);
        }
        return CommonResponse.create(500, "修改失败");
    }


    @Override
    public CommonResponse<Void> deleteDetail(Long id) {
        GhDetail ghDetail = new GhDetail();
        ghDetail.setId(id);
        ghDetail.setIsDelete(1);
        ghDetailMapper.updateByPrimaryKeySelective(ghDetail);
        return CommonResponse.success();
    }

}
